/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "AnimationManager.h"
#include "Animation.h"
#include "IsoException.h"

AnimationManager::AnimationManager() {

    // Checks to make sure all the graphics are loaded correctly
    if( !loadAllGraphics() ) {
        TRACE( "Error loading graphics" );
    }
}

AnimationManager::~AnimationManager() {
    // Releases all the textures
    for( int i = 0; i < 12; i++ ) {
        upLeft.at(i)->Release();
        upRight.at(i)->Release();
        downLeft.at(i)->Release();
        downRight.at(i)->Release();    
        bombsDownLeft.at(i)->Release();
        bombsDownRight.at(i)->Release();
        bombsUpLeft.at(i)->Release();
        bombsUpRight.at(i)->Release();
        feetDownLeft.at(i)->Release();
        feetDownRight.at(i)->Release();
        feetUpRight.at(i)->Release();
        feetUpLeft.at(i)->Release();
        if( i < 7 )
            explosion.at(i)->Release();
    }
	hitPointPowerUp.at(0)->Release();
	homingBombPowerUp.at(0)->Release();
	bombPowerUp.at(0)->Release();
	speedPowerUp.at(0)->Release();
}

bool AnimationManager::loadAllGraphics() {
    // Uses to store the names as they're loaded
    char filename[32];

    // Loads all 12 textures for the UpRight direction animation
    // from the grfx directory, pushes them all into the correct
    // vector
    for( int i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/UpRight%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {
            throw IsoException( "Error creating isoman upright textures" );
            return false;
        }
        upRight.push_back( texture );
    }

    // Loads all 12 textures for the UpLeft direction animation
    // from the grfx directory, pushes them all into the correct
    // vector
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/UpLeft%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating isoman upleft textures" );
            return false;
        }
        upLeft.push_back( texture );
    }

    // Loads all 12 textures for the DownRight direction animation
    // from the grfx directory, pushes them all into the correct
    // vector
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/DownRight%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating isoman downRight textures" );
            return false;
        }
        downRight.push_back( texture );
    }

    // Loads all 12 textures for the DownLeft direction animation
    // from the grfx directory, pushes them all into the correct
    // vector
    for(  i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/DownLeft%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating isoman DownLeft textures" );;
            return false;
        }
        downLeft.push_back( texture );
    }

    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/Bomb/BDL%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating Bomb DownLeft textures" );
            return false;
        }
        bombsDownLeft.push_back( texture );
    }

    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/Bomb/BDR%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating Bomb DownRight textures" );
            return false;
        }
        bombsDownRight.push_back( texture );
    }

    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/Bomb/BUR%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating Bomb UpRight textures" );
            return false;
        }
        bombsUpRight.push_back( texture );
    }
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/Bomb/BUL%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating Bomb UpLeft textures" );
            return false;
        }
        bombsUpLeft.push_back( texture );
    }

    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/BombFeet/FDL%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating BombFeet DownLeft textures" );
            return false;
        }
        feetDownLeft.push_back( texture );
    }
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/BombFeet/FDR%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating BombFeet DownRight textures" );
            return false;
        }
        feetDownRight.push_back( texture );
    }
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/BombFeet/FUR%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating BombFeet UpRight textures" );
            return false;
        }
        feetUpRight.push_back( texture );
    }
    for( i = 0; i < 12; i++ ) {
        sprintf( filename, "grfx/BombFeet/FUL%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating BombFeet UpLeft textures" );
            return false;
        }
        feetUpLeft.push_back( texture );
    }
    for( i = 0; i < 7; i++ ) {
        sprintf( filename, "grfx/Explosion/Exp%d.dds", i );
        if( FAILED( D3DXCreateTextureFromFile( 
                    g_dx->getDeviceHandle(), 
                    filename,
                    &texture ))) {

            throw IsoException( "Error creating Explosion textures" );
            return false;
        }
        explosion.push_back( texture );
    }
	sprintf( filename, "grfx/PowerUps/HP.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Hit Point Powerup texture" );
        return false;
    }
    hitPointPowerUp.push_back( texture );

	sprintf( filename, "grfx/PowerUps/HB.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Homing Bomb PowerUp texture" );
        return false;
    }
    homingBombPowerUp.push_back( texture );

	sprintf( filename, "grfx/PowerUps/Bomb.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Max Bombs PowerUp texture" );
        return false;
    }
    bombPowerUp.push_back( texture );

	sprintf( filename, "grfx/PowerUps/Power.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Increase Power PowerUp texture" );
        return false;
    }
    powerPowerUp.push_back( texture );

	sprintf( filename, "grfx/PowerUps/Radius.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Blast Radius PowerUp texture" );
        return false;
    }
    radiusPowerUp.push_back( texture );

	sprintf( filename, "grfx/PowerUps/Speed.dds" );
    if( FAILED( D3DXCreateTextureFromFile( 
                g_dx->getDeviceHandle(), 
                filename,
                &texture ))) {

        throw IsoException( "Error creating Speed PowerUp texture" );
        return false;
    }
    speedPowerUp.push_back( texture );

    // All loaded correctly, so return true;
    return true;
}

AnimationFrames* AnimationManager::getBombFeetAnimationFrames( int direction ) {
    if( direction == DIRECTION_UPLEFT )
        return &feetUpLeft;

    if( direction == DIRECTION_UPRIGHT )
        return &feetUpRight;

    if( direction == DIRECTION_DOWNRIGHT )
        return &feetDownRight;

    if( direction == DIRECTION_DOWNLEFT )
        return &feetDownLeft;
    
    return NULL;
}

AnimationFrames* AnimationManager::getBombAnimationFrames( int direction ) {

    if( direction == DIRECTION_UPLEFT )
        return &bombsUpLeft;

    if( direction == DIRECTION_UPRIGHT )
        return &bombsUpRight;

    if( direction == DIRECTION_DOWNRIGHT )
        return &bombsDownRight;

    if( direction == DIRECTION_DOWNLEFT )
        return &bombsDownLeft;
    
    return NULL;
}

AnimationFrames* AnimationManager::getCharacterAnimationFrames( int direction ) {

    // Constants defined in isobomb.h, so they return a pointer to the correct
    // animation frames
    if( direction == DIRECTION_UPLEFT )
        return &upLeft;

    if( direction == DIRECTION_UPRIGHT )
        return &upRight;

    if( direction == DIRECTION_DOWNRIGHT )
        return &downRight;

    if( direction == DIRECTION_DOWNLEFT )
        return &downLeft;

    return NULL;
}

AnimationFrames* AnimationManager::getExplosionAnimationFrames() {
    return &explosion;
}

AnimationFrames* AnimationManager::getHitPointTexture() {
	return &hitPointPowerUp;
}

AnimationFrames* AnimationManager::getHomingBombTexture() {
	return &homingBombPowerUp;
}

AnimationFrames* AnimationManager::getBombPowerUpTexture() {
	return &bombPowerUp;
}

AnimationFrames* AnimationManager::getPowerPowerUpTexture() {
	return &powerPowerUp;
}

AnimationFrames* AnimationManager::getRadiusPowerUpTexture() {
	return &radiusPowerUp;
}

AnimationFrames* AnimationManager::getSpeedPowerUpTexture() {
	return &speedPowerUp;
}