/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _BOMB_H_
#define _BOMB_H_

#include "IsoBomb.h"
#include "Entity.h"

/**
 * A bomb in the game that explodes and can hurt players that collide
 * with its blast.
 */
class Bomb : public Entity {

public:

    /**
     * Construct a bomb at a specified map location, blast size, and detonation
     * delay.
     *
     * @param initPos                   the initial map position
     * @param initDir                   the initial direction
     * @param size                      the blast size
     * @param delay                     the detonation delay
     */
    Bomb( const Rect& initPos, int initDir, float size, float delay );

    /**
     * Destructor
     */
    virtual ~Bomb();

    /**
     * Update the bomb's logic.
     *
     * @param dt                        the amount of time since the last frame
     * @returns                         true if this bomb has blown up
     */
    virtual bool update( float dt );

    /**
     * Draw the bomb on the screen using the specified anchor point.
     *
     * @param anchor                    the bomb's anchor point
     */
    virtual void draw( const Point& anchor );

    /**
     * Returns the current height.
     *
     * @returns                         the current height 
     */
    virtual float getHeight() const;

    /**
     *	Handles the collision between 2 entities by finding what type of
     *	entities they are and then handling the different types of collisions;
     *
     *  @param otherEntity       the entity to check collision with
     */
    virtual void doCollision( Entity* otherEntity );

protected:

    /**
     * Tells the bomb to create explosions.
     */
    void blowUp();

protected:

    /**
     * The size of this bomb's explosion.
     */
    float blastSize;

    /**
     * The time at which this bomb explodes.
     */
    float explodeTime;

    /**
     * The bomb's current height.
     */
    float height;

    /**
     * Number of particles spawned so far.
     */
    int pSoFar;

    /**
     * Amount of time lived so far.
     */
    float timeLived;

protected:
    /**
     * Bomb is not defaultly constructable.
     */
    Bomb();
};

#endif