/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef DRAWABLE_H_KPV401
#define DRAWABLE_H_KPV401

/**
 * Drawable is an interface class representing a drawable object on the Map.
 * A Drawable object must have a position.
 */
class Drawable {
public:
    virtual ~Drawable() {
    }

    /**
     * Draw this object on the screen, given the anchor point on the screen,
     * which is defined as the center of the object's contact with the ground.
     */
    virtual void draw( const Point& anchor ) = 0;

    /**
     * Get the current position on the map.
     *
     * @returns the current position on the map
     */
    virtual const Rect& getPosition() const = 0;

    /**
     * Returns the height of the object.  If the returned number is negative
     * (the default implementation), the object's height is taken to be the
     * tile's height when it is drawn.
     * 
     * If the number is 0.0f or greater, the height given is taken to be the
     * absolute height, and the tile's height will not be taken into
     * consideration.
     *
     * The default implementation of getHeight returns a negative number.
     */
    virtual float getHeight() const {
        return -1.0f;
    }

    /**
     * A method that returns true if the Drawable will be drawing
     * pre-transformed (XYZRHW) and pre-lit vertices, and its anchor needs
     * transforming due to viewport changes.
     */
    virtual bool needsTransform() const {
        return false;
    }

protected:
    Drawable() {
    }
private:
};

#endif
