/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef EFFECT_H_KPR241
#define EFFECT_H_KPR241

#include "Drawable.h"
#include "Rect.h"
class Point;

/**
 * An Effect is an object in the game that is drawn, but does not "have a
 * solid existance" like an Entity does -- meaning it is not checked for
 * collisions and therefore is not an essential part of the gameplay logic.
 *
 * Effects are updated, drawn, and can die.
 */
class Effect : public Drawable {
public:
    /**
     * Constructor
     */
    Effect();

    /**
     * Destructor
     */
    virtual ~Effect();

    /**
     * Update the logic for the effect.
     *
     * @param dt the amount of time since the last frame
     * @returns true if this Effect should be removed from the game
     */
    virtual bool update( float dt )= 0;

    /**
     * Draw this Effect on the screen, given the anchor point on the screen,
     * which is defined as the center of the Effect's contact with the ground.
     */
    virtual void draw( const Point& anchor ) = 0;

    /**
     * Get the current position on the map.
     *
     * @returns the current position on the map
     */
    virtual const Rect& getPosition() const = 0;
};

#endif
