/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef EXPLOSION_H_XYZ432
#define EXPLOSION_H_XYZ432

#include "IsoBomb.h"
#include "Entity.h"

/**
 *  Takes care of the explosion for each bomb, it takes the blast size and
 *  position of the start of the explosion and creates the whole blast from there
 */
class Explosion : public Entity {

public:

    /**
     * Default constructor
     */
    Explosion();

    /**
     * Construct a explosion at a specified map location, blast size
     *
     * @param initPos                   the initial map position
     */
    Explosion( const Rect& initPos, float initHeight );

    /**
     * Destructor
     */
    virtual ~Explosion();

    /**
     * Update the explosion.
     *
     * @param dt                        the amount of time since the last frame
     * @returns                         true if this explosion is done
     */
    virtual bool update( float dt );

    /**
     * Draw the explosion on the screen using the specified anchor point.
     *
     * @param anchor                    the explosions's anchor point
     */
    virtual void draw( const Point& anchor );

	virtual float getHeight() const;

	virtual void setScaleSize( float s );

	virtual void setOffsets( int x, int y );

private:

	float height;

};

#endif