/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _KEYBOARD_H_
#define _KEYBOARD_H_

/**
 * Reads from the keyboard using DirectInput.
 * Taken From: The Zen of Direct3D Game Programming (Peter Walsh)
 *
 */
class Keyboard {

public:
    /**
     * Constructor
     *
     */
	Keyboard();

    /**
     * Destructor
     *
     */
	~Keyboard();

public:

    /**
     * Initializes the keyboard and ties to the window specified
     *
     * @param   hWnd        the window handle
     * @returns             the result
     */
	HRESULT initialize( HWND hWnd );

    /**
     * Returns TRUE if the key specified is currently held down
     *
     * @param   Key         the key to test
     * @returns             TRUE if the key specified is currently held down
     */
	bool isKeyDown( int Key );

protected:

    /**
     * A DirectInput device
     *
     */
	LPDIRECTINPUTDEVICE8 m_pKeyDev;

    /**
     * A key buffer
     *
     */
	char m_KeyBuffer[256];

    /**
     * True if the keyboard has been initialized
     *
     */
	BOOL m_bInitialized;

};

#endif