/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef POINT_H_KWER341
#define POINT_H_KWER341

#include "IsoBomb.h"

/**
 * The Point class is a point using floats that is typically a location in the
 * world coordinate and not the screen coordinate space.
 *
 * Note: This class was brought in from another project Jason has done.
 */
class Point {
public:
    float x;
    float y;

    /**
     * Default ctor.
     */
    Point() : x(0.0f), y(0.0f) {}

    /**
     * Constructor allowing for initialization.
     */
    Point(float x2, float y2) : x(x2), y(y2) {}

    /**
     * Reinitializes the Point.
     */
    void init(float x2, float y2) {
        x = x2;
        y = y2;
    }

    /**
     * Returns the distance from the origin, or Point( 0.0f, 0.0f ).
     */
    float distFromOrigin() const {
        return (float)(fabs(sqrt(x*x+y*y)));
    }

    /**
     * Returns the distance between this Point and the given Point.
     */
    float distTo(const Point& other) const {
        return (float)(fabs(sqrt( sq(other.x-x) + sq(other.y-y) )));
    }

    /**
     * Finds angle between a vector on the positive x axis from this Point
     * and the line segment from this Point to the given Point.
     */
    float thetaTo(const Point& other) const {
        return (float)(atan2(other.y-y, other.x-x));
    }

    bool operator<( const Point& rhs ) const {
        if( x == rhs.x ) {
           return y < rhs.y;
        }
        else {
           return x < rhs.x;
        }
    }

protected:
private:
};

#endif
