/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "Game.h"
#include "RespawnPowerup.h"
#include "Powerup.h"
#include "Vector3D.h"
#include "Point3D.h"
#include "Particle.h"
#include "SoundSystem.h"

const int NUM_PARTICLES = 25;

RespawnPowerup::RespawnPowerup( Powerup* p, float timeToRespawn )
: p(p), timeLeft( timeToRespawn ), soundPlayed( false ) {
    assert( p != NULL );
}

RespawnPowerup::~RespawnPowerup() {
    delete p;
}

bool RespawnPowerup::update( float dt ) {
    timeLeft -= dt;

    if ( !soundPlayed ) {
        if ( timeLeft <= 0.9f ) {
            g_snd->playPowerupRespawnSound();
            soundPlayed = true;
        }
    } else if ( timeLeft <= 0.0f )
        doRespawn();

    return ( p == NULL );
}

void RespawnPowerup::doRespawn() {
    assert( p != NULL );

    //Spawn some particles along with this
    Point center( p->getPosition().getCenter() );
    for ( int i = 0; i < NUM_PARTICLES; ++i ) {
        Particle* part = new Particle(
            Point3D( center.x, center.y, p->getHeight() + 2.0f ),
            Vector3D( getrndf( 1.0f, -1.0f ), getrndf( 1.0f, -1.0f ), getrndf( 10.0f ) ),
            getrndf( 2.0f, 1.0f ) );
        part->setColor( D3DCOLOR_ARGB( 128, 78, 129, getrnd( 255, 207 ) ) );
        part->setBounceFactor( 0.0f );
        g_game->addEffect( part );
    }

    g_game->addEntity( p );
    p = NULL;
}
