/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef RESPAWNPOWERUP_H_PQM302
#define RESPAWNPOWERUP_H_PQM302

#include "GameLogic.h"

class Powerup;

/**
 * The RespawnPowerup class waits for the given amount of time.  After the time
 * has passed, the Powerup given to the object is inserted into the game's list
 * of Entities.
 *
 * The difference between RespawnPowerup and RespawnEntity is this class starts
 * playing the respawn sound before the Powerup reappears (this is because of
 * how the sound is designed), and it also creates particles when the Powerup
 * reappears.
 *
 * RespawnPowerup is not default-constructable or copyable.
 */
class RespawnPowerup : public GameLogic {
public:
    /**
     * Constructs a new RespawnPowerup.  Adds the passed Powerup p to the Game
     * after timeToRespawn has passed.  After p is back into the game, the
     * update method will return true.
     */
    RespawnPowerup( Powerup* p, float timeToRespawn );

    /**
     * Cleans up this object.  If this object is destructed, it does delete
     * the passed in Powerup.  This should only happen if the program exits
     * before the respawn event takes place.
     */
    virtual ~RespawnPowerup();

    /**
     * Waits to respawn this object's Powerup.  After the Powerup has been added
     * back into the game, update returns true.
     */
    bool update( float dt );

private:
    RespawnPowerup();
    RespawnPowerup( const RespawnPowerup& other );
    RespawnPowerup& operator = ( const RespawnPowerup& rhs );

    void doRespawn();

    float timeLeft;
    Powerup* p;
    bool soundPlayed;
};

#endif //#ifndef RESPAWNPOWERUP_H_PQM302
