/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "TextureManager.h"

TextureManager::TextureManager( std::string directory, std::string extension,
                                GraphicsDevice device ) {
    using namespace std;
    
    int currTex = 0;
    for (;;) {
        Texture tex;
        ostringstream filename;

        //Load the top texture
        filename << directory << '/' << currTex << '.' << extension;
        TRACE1( "Attempting to load texture: %s", filename.str().c_str() );

        if ( FAILED( D3DXCreateTextureFromFile( device, filename.str().c_str(), &(tex) ) ) )
            break;

        textures.push_back( tex );
        ++currTex;
    }
}

TextureManager::~TextureManager() {
    TVectorIter iter = textures.begin();
    while ( iter != textures.end() ) {
        (*iter)->Release();
        ++iter;
    }
}

int TextureManager::numTextures() const {
    return (int)textures.size();
}

Texture TextureManager::getTexture( int idx ) const {
    assert( 0 <= idx && idx < numTextures() );
    return textures[idx];
}
