/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "ThrownBomb.h"
#include "Game.h"
#include "Tile.h"
#include "Map.h"
#include "Explosion.h"

ThrownBomb::ThrownBomb() : owner(NULL), hasLanded(false) {

}

ThrownBomb::ThrownBomb( const Rect& initPos, int initDir, float size, 
                        float delay, float power, Character* c ) :
    Bomb( initPos, initDir, size, delay ), owner(c), hasLanded(false) {

    vect.initRect( power * 5.0f, power * 20.0f );

    height = g_game->getMap()->getRectHeight(pos) + 3.0f;
}

ThrownBomb::~ThrownBomb() {

}

bool ThrownBomb::update( float dt ) {
    assert( owner != NULL );

    // If height is less than 0, it has landed
    if( !hasLanded ) {
        int tileHeight = g_game->getMap()->getRectHeight( pos );

        // Check to see if the bomb has landed
        if( ceil(height) == tileHeight ) {
            hasLanded = true;
            height = (float)tileHeight;
        }
        else {
            Rect newPos( pos );

            // Subtract gravity from the y magnitude
            Vector grav;
            grav.initRect( 0.0f, -50.0f * dt );
            vect.add( grav );
            // Update the height of the bomb
            height += vect.getMagy() * dt;

            // Update the position of the bomb
            switch( direction ) {
            case DIRECTION_DOWNRIGHT:
                newPos.x += vect.getMagx() * dt;
                break;
            case DIRECTION_UPLEFT:
                newPos.x -= vect.getMagx() * dt;
                break;
            case DIRECTION_UPRIGHT:
                newPos.y -= vect.getMagx() * dt;
                break;
            case DIRECTION_DOWNLEFT:
                newPos.y += vect.getMagx() * dt;
                break;
            }

            // Prevent bomb from going outside the map
            if( !g_game->getMap()->isLocationOutside(newPos) ) {
                tileHeight = g_game->getMap()->getRectHeight( newPos );

                // Bounce if about to hit a wall
                if( height < tileHeight ) {
                    reverseDirection();
                    // Mutate the vector a little to give it a more desirable bounce
                    vect.initRect( vect.getMagx() * 0.75f, vect.getMagy() * 2.0f );
                }
                else {
                    // Set the new position only if it's on the map and isn't
                    // about to bounce.
                    pos = newPos;
                }
            }
        }
    }

    // Update the bomb's timer
    bool explode = Bomb::update(dt);
    // Decrement the owner's bomb counter when this bomb explodes.
    if( explode) {
        owner->decBombCounter();        
    }

    return explode;
}

void ThrownBomb::draw( const Point& anchor ) {
    Bomb::draw( anchor );
}

void ThrownBomb::doCollision( Entity* otherEntity ) {
    Bomb::doCollision( otherEntity );

	Character* c = dynamic_cast<Character*>(otherEntity);
	if( c != NULL && !hasLanded ) {
		// Bounce off the character's head
		vect.initRect( vect.getMagx(), 15.0f );
        reverseDirection();
    }
}