/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _TILE_H_
#define _TILE_H_

#include "IsoBomb.h"

class Drawable;

/**
 * A node in the render list.
 */
struct ObjListNode {
    /**
     * The object to draw.
     */
    Drawable* obj;

    /**
     * The object's screen offset within the tile.  This offset is based as if
     * the object was resting on the ground.  It is stored this way for
     * purposes of draw order.  Height is taken into account at draw time.
     */
    Point scrOffset;

    /**
     * An operator that is used for sorting the object list by screen y offset
     * (draw order).
     */
    bool operator >= ( const ObjListNode& rhs ) const {
        return ( scrOffset.y >= rhs.scrOffset.y );
    }
};

/**
 * A tile on the map
 */
struct Tile {

    /**
     * The height of the tile
     */
    int height;

    /**
     * The texture to use on the top of the tile
     */
    LPDIRECT3DTEXTURE8 topTex;

    /**
     * The texture to use on the sides of the tile
     */
    LPDIRECT3DTEXTURE8 sideTex;

    /**
     * A std::list of objects to draw.
     */
    typedef std::list< ObjListNode > DrawList;

    /**
     * An iterator over a list of entities.
     */
    typedef DrawList::iterator DrawListIter;

    /**
     * A list of objects in this tile to render.
     */
    DrawList objs;
};

#endif
