/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"

Timer::Timer() {
    time_Frequency = 0;
    time_StartTime = 0;
    time_FrameCount = 0;
    time_FrameRate = 0;
    time_LastFrame = 0;

    QueryPerformanceFrequency( (LARGE_INTEGER*)&time_Frequency );
    if( time_Frequency == 0 ) {
        TRACE( "Unable to create high performance counter" );
    }
}

Timer::~Timer() {
    time_Frequency = 0;
    time_StartTime = 0;
    time_FrameCount = 0;
    time_FrameRate = 0;
    time_LastFrame = 0;
}

void Timer::setStartTime() {
    QueryPerformanceCounter( (LARGE_INTEGER*)&time_StartTime );
}

float Timer::getSecondsSinceStart() {
    INT64 curr;
    QueryPerformanceCounter( (LARGE_INTEGER*)&curr );
    curr -= time_StartTime;
    //I was (and still am) debating between doubles and floats.  It is possible
    //for the timer frequency to be large enough that we lose a few digits of
    //precision, but floats should be faster.
    float ret = (float)curr;
    ret /= (float)time_Frequency;
    return (float)ret;
}

float Timer::getNumTicksPerMs() {
    return((float)time_Frequency / 1000.0f );
}

void Timer::calcFrameCount() {
    INT64 NewCount = 0;
    static INT64 LastCount = 0;
    INT64 Difference = 0;

    QueryPerformanceCounter( (LARGE_INTEGER*) &NewCount );

    if( NewCount == 0 ) {
        TRACE( "No high performance counter" );
    }

    time_FrameCount++;

    Difference = NewCount - LastCount;

    if( Difference >= time_Frequency ) {
        time_FrameRate = time_FrameCount;
        time_FrameCount = 0;
        LastCount = NewCount;
    }
}

int Timer::getFrameRate() {
    return time_FrameRate;
}

float Timer::getFrameDt() {
    float curr = getSecondsSinceStart();
    float dt = curr - time_LastFrame;
    time_LastFrame = curr;
    return dt;
}