/**
* Super IsoBomb
*
* Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
*/

#ifndef _VECTOR_H_
#define _VECTOR_H_

#include "IsoBomb.h"

/**
* The Vector class represents a 2D vector.  Vectors support addition, polar
* and rectangular coordinates, addition to vectors, addition to Rect s and
* Point s.  Multiplication and so on.
*
* Note:  This class like Rect and Point come from classes Jason has used in
* four different projects so they are well tested and understood, for this
* reason we decided to use them instead of the D3DX/Win32 classes/structs.
*/
class Vector {
public:
    Vector() : magx(0), magy(0), theta(0), mag(0) {}
    /**
     * Initalizes this vector using rectangular coordinates.
     */
    Vector(float magx2, float magy2) {initRect(magx2, magy2);}

    /**
     * Initalizes this vector using rectangular coordinates.
     */
    void initRect(float magx2, float magy2);

    /**
     * Initalizes this vector using polar coordinates.
     */
    void initPolar(float theta2, float mag2);

    /**
     * Returns the x component of this vector.
     */
    float getMagx() const {return magx;}

    /**
     * Returns the y component of this vector.
     */
    float getMagy() const {return magy;}

    /**
     * Returns the angle counterclockwise off the positive x axis.
     */
    float getTheta() const {return theta;}

    /**
     * Returns the magnitude of this vector.
     */
    float getMag() const {return mag;}

    /**
     * Rotates the vector counterclockwise by the given rotation.
     */
    void rotate(float rotation);

    /**
     * Vector addition.  This is equivalent to operator +=.
     */
    void add(const Vector& other);  //Vector addition

    /**
     * Adds a scalar value to the magnitude of this vector.
     */
    void add(float scalar);        //add to magnitude

    /**
     * Clips the magnitude to the given maximum value.
     */
    void clip(float max);          //clips mag to max

    //void clipTheta(double newTheta, double tolerance);
    //clips theta to newTheta if it is within tolerance

    /**
     * Multiplies the magnitude by the given scalar.
     */
    void mult(float scalar);       //multiplies to total magnitude

    /**
     * Adds the x and y components of this vector to the Point.  This has the
     * effect of "moving" the point.
     */
    void addTo(Point& other) const {
        other.x += magx;
        other.y += magy;
    }

    /**
     * Adds the x and y components of this vector to the Rect.  This has the
     * effect of "moving" the Rect.
     */
    void addTo(Rect& other) const {
        other.x += magx;
        other.y += magy;
    }

    /**
     * Sets the value of theta
     */
    void setTheta( float rot );

private:
    float magx, magy;
    float theta, mag;
};

#endif
