/**
* Super IsoBomb
*
* Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
*/

#ifndef VECTOR3D_JIP695
#define VECTOR3D_JIP695

#include "Point3D.h"

/**
 * The Vector class represents a 3D vector.  Vectors support addition,
 * spherical coordinates, addition to vectors, addition to Point3D objects,
 * and so on.
 */
class Vector3D {
public:
    Vector3D() : magx(0.0f), magy(0.0f), magz(0.0f) {}

    /**
     * Initalizes this vector using rectangular coordinates.
     */
    Vector3D( float magx, float magy, float magz ) {
        initRect( magx, magy, magz );
    }

    /**
     * Destructor
     */
    ~Vector3D() {}

    /**
     * Initalizes this vector using rectangular coordinates.
     */
    void initRect( float magx, float magy, float magz );

    /**
     * Initalizes this vector using spherical coordinates.
     * @param r     the distance (magnitude) of the vector.
     * @param theta the angle on the x-y plane off the positive x axis (angle of ascension)
     * @param phi   the angle off the positive z axis (angle of declination).
     *              an angle of 0 points along the positive z axis, and angle
     *              of PI/2 in the x-y plane, and PI points along the negative
     *              z axis.
     */
    void initSphere( float r, float theta, float phi );

    /**
     * Returns the x component of this vector.
     */
    float getMagx() const {return magx;}

    /**
     * Returns the y component of this vector.
     */
    float getMagy() const {return magy;}

    /**
     * Returns the z component of this vector.
     */
    float getMagz() const {return magz;}

    /**
     * Multiplies the magnitude by the given scalar.
     */
    void mult( float scalar );

    /**
     * Multiplies only the X magnitude.
     */
    void multX( float scalar );

    /**
     * Multiplies only the Y magnitude.
     */
    void multY( float scalar );

    /**
     * Multiplies only the Z magnitude.
     */
    void multZ( float scalar );

    /**
     * Vector addition.  This is equivalent to operator +=.
     */
    void add(const Vector3D& other);

    /**
     * Adds the vector rhs to this vector.
     */
    Vector3D& operator += ( const Vector3D& rhs );

    /**
     * Adds the components of this vector to the given point.  This has the
     * effect of "moving" the point.
     */
    void addTo( Point3D& other ) const {
        other.x += magx;
        other.y += magy;
        other.z += magz;
    }

private:
    float magx, magy, magz;
};

#endif
