/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"

Window::Window() {
    TRACE("Created Game Window Object");
}

Window::~Window() {
    CoUninitialize(); //to clean up windows a little better. not required.
    TRACE("Terminating Game Window Object");
    //nothing we are killing this structure...
}

void Window::registerClass(HINSTANCE hInstance, const char* ATitle) {
    WNDCLASSEX wcex;
    wcex.cbSize = sizeof(WNDCLASSEX);

    wcex.style = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc = (WNDPROC)GlobalWndProc;
    wcex.cbClsExtra = 0;
    wcex.cbWndExtra = 0;
    wcex.hInstance = hInstance;
    wcex.hIcon = LoadIcon( NULL, IDI_APPLICATION );
    wcex.hIconSm = LoadIcon( NULL, IDI_HAND );
    wcex.hCursor = LoadCursor( NULL, IDC_ARROW );
    wcex.hbrBackground = (HBRUSH)GetStockObject(DKGRAY_BRUSH);
    wcex.lpszMenuName = 0;
    wcex.lpszClassName = ATitle;

    RegisterClassEx(&wcex);
}

BOOL Window::initInstance(HINSTANCE hInst, int nCmdShow, const char* ATitle, const char* WTitle) {

    hInstance = hInst;

    int posX = 0;
    int posY = 0;

    DWORD ws = WS_OVERLAPPEDWINDOW;
    RECT rc;
    rc.left = posX;
    rc.top = posY;
    rc.right = g_winSizeX;
    rc.bottom = g_winSizeY;
    AdjustWindowRect(&rc, ws, FALSE);

    hWnd = CreateWindowEx( 
        NULL,
        ATitle,
        WTitle,
        ws,
        posX,
        posY,
        rc.right - rc.left + posX,
        rc.bottom - rc.top + posY,
        NULL,
        NULL,
        hInstance,
        NULL
        );

    if( !hWnd ) {
        TRACE("GOT A FALSE WINDOW - BAD NEWS");
        return FALSE;
    }

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    return TRUE;
}


const HWND & Window::getHwnd() const {
    return hWnd;
}

const HINSTANCE & Window::get_hInstance() const {
    return hInstance;
}

/*
int Window::getTop() const {
    return oldPosition.top;
}

int Window::getLeft() const {
    return oldPosition.left;
}
*/

void Window::setWindowed( BOOL newWindowed ) {
    windowed = newWindowed;
}

BOOL Window::getWindowed() const {
    return windowed;
}

bool Window::toggleWindowed() {
    windowed = !windowed;

    if( windowed ) {
        if ( FAILED( g_dx->resetD3D() ) )
            return true;
        //const RECT& o( oldPosition );
        MoveWindow( hWnd, oldPosition.left, oldPosition.top, oldPosition.right-oldPosition.left, oldPosition.bottom-oldPosition.top, TRUE );
    } else {
        // Saves the position of the window
        GetWindowRect( hWnd, &oldPosition );
        if ( FAILED( g_dx->resetD3D() ) )
            return true;
    }

    return false;
}
