/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "Virus.h"
#include "Powerup.h"
#include "Character.h"
#include "ConfigFile.h"
#include "SoundSystem.h"

float Virus::lifeTime = 20.0f;
float Virus::slowSpeed = 2.0f;
float Virus::delayTime = 20.0f;
float Virus::weakPower = 0.035f;

void Virus::initVars( ConfigFile& f ) {
    f.getVar( "Virus", "lifeTime", lifeTime, 20.0f );
    f.getVar( "Virus", "slowSpeed", slowSpeed, 2.0f );
    f.getVar( "Virus", "delayTime", delayTime, 20.0f );
    f.getVar( "Virus", "weakPower", weakPower, 0.035f );
}

Virus::Virus() : countdown(0.0f), red(255.0f), 
                increaseColor(false), throwTime(0.0f) {

}

Virus::Virus( char virType, Character* c ) {
    init( virType, c, lifeTime );
}

Virus::Virus( char virType, Character* c, float timeLeft ) {
    init( virType, c, timeLeft );
}


void Virus::init( char virType, Character* c, float timeLeft ) {
    type = virType;
    countdown = timeLeft;
    host = c ;
    red = 255.0f;
    increaseColor = false;
    throwTime = 0.0f; 

    CharacterProps origProps = host->getProps();

    // Blank props by default
    props.blastSize = 0.0f;
    props.detonationDelay = 0.0f;
    props.homingSpeed = 0.0f;
    props.hp = 0;
    props.maxBombs = 0;
    props.maxPower = 0.0f;
    props.numHoming = 0;
    props.powerRate = 0.0f;
    props.speed = 0.0f;

    switch( type ) {
    case SLOW_VIRUS:
        props.speed = -(origProps.speed - slowSpeed);
        g_snd->playSlowdownVoice();
        break;
    case NOBOMBS_VIRUS:
        props.maxBombs = -origProps.maxBombs;
        props.numHoming = -origProps.numHoming;
        g_snd->playBombConstipationVoice();
        break;
    case BOMBARRHEA_VIRUS:
        throwTime = countdown - 1.0f;
        g_snd->playBombarrheaVoice();
        break;
    case WEAKTHROW_VIRUS:
        props.maxPower = -(origProps.maxPower - 0.35f);
        g_snd->playFrailtyVoice();
        break;
    case DELAY_VIRUS:
        props.detonationDelay = -(origProps.detonationDelay - 20.0f);
        g_snd->playSlowBombsVoice();
        break;
    case SHRINK_VIRUS:
        props.speed = -(origProps.speed - slowSpeed);
        props.maxPower = -(origProps.maxPower - 0.35f);
        host->setShrink( true );
        g_snd->playShrinkVoice();
        break;
    default:
        assert( false );
    }


    host->affectMe( props );
}

Virus::~Virus() {

}

bool Virus::update( float dt ) {
    countdown -= dt;

    // Update red value
    if( increaseColor ) {
        red += (dt * RED_CHANGE_RATE);
        if( red >= MAX_RED_VALUE ) {
            increaseColor = false;
            red = MAX_RED_VALUE;
        }
    }
    else {
        red -= (dt * RED_CHANGE_RATE);
        if( red <= MIN_RED_VALUE ) {
            increaseColor = true;
            red = MIN_RED_VALUE;
        }
    }

    // If bombarrhea, throw bombs randomly
    if( type == BOMBARRHEA_VIRUS ) {
        host->chargeThrow( dt );
        if( host->isCharging() && (countdown <= throwTime) ) {
            host->throwBomb();
            throwTime = countdown - (rand() % 5) + 1.0f;
        }
    }

    if( countdown <= 0.0f ) {
        // Get inverse of the props
        props.blastSize = -props.blastSize;
        props.detonationDelay = -props.detonationDelay;
        props.homingSpeed = -props.homingSpeed;
        props.hp = -props.hp;
        props.maxBombs = -props.maxBombs;
        props.maxPower = -props.maxPower;
        props.numHoming = -props.numHoming;
        props.powerRate = -props.powerRate;
        props.speed = -props.speed;

        host->affectMe( props );

        if( type == SHRINK_VIRUS ) {
            host->setShrink( false );
        }

        return true;
    }

    return false;
}

int Virus::getRedValue() const {
    return (int)red;
}

Virus* Virus::getCloneFor( Character* c ) const {
    return new Virus( type, c, countdown );
}

int Virus::getVirusType() const {
    return type;
}